/*  Canon Bubble Jet Print Filter.
 *  Copyright CANON INC. 2001
 *  All Rights Reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * NOTE:
 *  - As a special exception, this program is permissible to link with the
 *    libraries released as the binary modules.
 *  - If you write modifications of your own for these programs, it is your
 *    choice whether to permit this exception to apply to your modifications.
 *    If you do not wish that, delete this exception.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef ENABLE_NLS
#  include <libintl.h>
#endif

#include <gtk/gtk.h>
#ifdef	USE_LIB_GLADE
#	include <glade/glade.h>
#endif
#include <stdio.h>

#include "bjuidefs.h"
#include "msgtable.h"


// BJ library dir
static char* bjlibdir = "/usr/lib/bjlib";


int InitDataBase(char* model_name)
{
	short size_value;

	if( model_name )
		g_model_id = ModelNameToID(model_name);
	else
		g_model_id = CND_BJF850;

	g_bjlibdir = bjlibdir;

	// Retrieve name information.
	if( CNCL_GetDBName(&g_uidb.nameinfo, (void *)g_bjlibdir, g_model_id) < 0 )
	{
		printf("Error: CNCL_GetDBName\n");
		return -1;
	}

	// Obtain the size of Internal Database.
	g_uidb.dbsize = CNCL_GetDBSize(&g_uidb.nameinfo, (void*)g_bjlibdir);

	// Allocate Internal DB Area.
	g_uidb.lpdbTop = (LPCNCLDB)g_malloc(sizeof(CNCLDB) * g_uidb.dbsize);
	g_uidb.modelID = g_model_id;

	// Initialize database values with CNCL.
	CNCL_GetEntries(&g_uidb.nameinfo, (void*)g_bjlibdir, g_uidb.lpdbTop);

	// Then get Defaults.
	CNCL_GetDefaults(&g_uidb.nameinfo,
				(void*)g_bjlibdir, g_uidb.lpdbTop, g_uidb.dbsize);

	// At last, call MODESET.
	CNCL_InitMenulink(&g_uidb.nameinfo,
				(void*)g_bjlibdir, g_uidb.lpdbTop, g_uidb.dbsize);

	// Get min and max paper size.
	CNCL_GetMargin(&g_uidb.nameinfo,
		(void*)g_bjlibdir, g_uidb.lpdbTop, &g_bj_paper_size, g_uidb.dbsize);

	// If the default unit is "inch", change the media size to "letter".
	if( g_unit_inch == TRUE )
		UpdateMenuLink(CNCL_PAPERSIZE, CND_SIZE_LETTER);

	// Set the user paper size into globals.
	size_value = GetCurrentnValue(CNCL_PAPERSIZE);

	g_user_paper_size[YOKO] = GetPaperWidth(size_value);
	g_user_paper_size[TATE] = GetPaperHeight(size_value);

#ifdef	DEBUG_DUMP_DATABASE
	DumpDataBase(-1);
#endif

	return 0;
}

void FreeDataBase()
{
	g_free(g_uidb.lpdbTop);
}

static
long GetPaperSizeIndex(short value)
{
	int index = 0;

	while( PaperTbl[index][PSID] != -1 )
	{
		if( PaperTbl[index][PSID] == value )
			return index;

		index++;
	}
	return 0;
}

long GetPaperWidth(short value)
{
	if( value == CND_SIZE_USER )
		return g_user_paper_size[YOKO];
	else
		return PaperTbl[GetPaperSizeIndex(value)][YOKO];
}

long GetPaperHeight(short value)
{
	if( value == CND_SIZE_USER )
		return g_user_paper_size[TATE];
	else
		return PaperTbl[GetPaperSizeIndex(value)][TATE];
}

int ModelNameToID(char* name)
{
	int i = 0;

	while( g_model_key[i] != NULL )
	{
		if( strcmp(name, LookupText(g_keytext_list, g_model_key[i])) == 0 )
			return g_model_value[i];
		i++;
	}
	return -1;
}

char* GetModelName()
{
	int i = 0;

	while( g_model_key[i] != NULL )
	{
		if( g_model_value[i] == g_model_id )
			return LookupText(g_keytext_list, g_model_key[i]);

		i++;
	}
	return NULL;
}

int GetValueAndNameArray(short id, short** value, char*** name)
{
	int count;

	switch( id )
	{
	case CNCL_MEDIATYPE:	// Media Type.
		*value = g_media_value;
		*name  = g_media_name;
		count = sizeof(g_media_value) / sizeof(short);
		break;

	case CNCL_MEDIASUPPLY:	// Media Supply.
		*value = g_supply_value;
		*name  = g_supply_name;
		count = sizeof(g_supply_value) / sizeof(short);
		break;

	case CNCL_ENHBK:		// Black Enhancement.
		*value = g_bkenh_value;
		*name  = g_bkenh_name;
		count = sizeof(g_bkenh_value) / sizeof(short);
		break;

	case CNCL_PAPERSIZE:	// Paper Size.
		*value = g_size_value;
		*name  = g_size_name;
		count = sizeof(g_size_value) / sizeof(short);
		break;

	case CNCL_MESS_QUALITY:	// Message for Quality.
		*value = g_quality_msg_value;
		*name  = g_quality_msg_name;
		count = sizeof(g_quality_msg_value) / sizeof(short);
		break;

	case CNCL_MESS_THICK:	// Message for Thickness.
		*value = g_thickness_msg_value;
		*name  = g_thickness_msg_name;
		count = sizeof(g_thickness_msg_value) / sizeof(short);
		break;

	case CNCL_MESS_PAPERLOAD:	// Message for Preload.
		*value = g_preload_msg_value;
		*name  = g_preload_msg_name;
		count = sizeof(g_preload_msg_value) / sizeof(short);
		break;

	case CNCL_MESS_RESOLUTION:	// Message for Resolution.
		*value = g_resolution_msg_value;
		*name  = g_resolution_msg_name;
		count = sizeof(g_resolution_msg_value) / sizeof(short);
		break;

	case CNCL_MESS_SMLIB:		// Message for Smoothing.
		*value = g_smoothing_msg_value;
		*name  = g_smoothing_msg_name;
		count = sizeof(g_smoothing_msg_value) / sizeof(short);
		break;

	case CNCL_DITHER_PAT:		// Halftoning.
		*value = g_halftoning_value;
		*name  = g_halftoning_name;
		count = sizeof(g_halftoning_value) / sizeof(short);
		break;

	case CNCL_BANNER:			// Banner.
		*value = g_banner_value;
		*name  = g_banner_name;
		count = sizeof(g_banner_value) / sizeof(short);
		break;

	case CNCL_CARTRIDGE:		// Cartridge.
		*value = g_cartridge_value;
		*name  = g_cartridge_name;
		count = sizeof(g_cartridge_value) / sizeof(short);
		break;

	case CNCL_INPUT_GAMMA:		// Input gamma.
		*value = g_input_gamma_value;
		*name  = g_input_gamma_name;
		count = sizeof(g_input_gamma_value) / sizeof(short);
		break;

	default:
		*value = NULL;
		*name  = NULL;
		count = 0;
		break;
	}

	return count;
}

char *ValueToKey(short id, short value)
{
	short *value_array;
	char **key_array;
	int count;
	int i;

	count = GetValueAndNameArray(id, &value_array, &key_array);

	for( i = 0 ; i < count ; i++ )
	{
		if( value_array[i] == value )
			break;
	}
	return (i < count)? key_array[i] :  "";
}

short KeyToValue(short id, char *name)
{
	short *value_array;
	char **key_array;
	int count;
	int i;

	count = GetValueAndNameArray(id, &value_array, &key_array);

	for( i = 0 ; i < count ; i++ )
	{
		if( strcmp(key_array[i], name) == 0 )
			break;
	}
	return (i < count)? value_array[i] : -1;
}

char *ValueToName(short id, short value)
{
	char *name = ValueToKey(id, value);

	if( *name != '\0' )
		name = LookupText(g_keytext_list, name);

	return name;
}

short NameToValue(short id, char *name)
{
	short *value_array;
	char **key_array;
	int count;
	int i;

	count = GetValueAndNameArray(id, &value_array, &key_array);

	for( i = 0 ; i < count ; i++ )
	{
		if( strcmp(LookupText(g_keytext_list, key_array[i]), name) == 0 )
			break;
	}
	return (i < count)? value_array[i] : -1;
}

GList *GetComboList(short id)
{
	LPCNCLDB lpdb = g_uidb.lpdbTop;
	GList *glist = NULL;
	char *str;
	short i;

	for( i = 0 ; i < g_uidb.dbsize ; i++, lpdb++ )
	{
		if( lpdb->nObjectID == id )
		{
			str = ValueToName(lpdb->nObjectID, lpdb->nValue);

			if( *str != '\0' && lpdb->disable == 0 )
				glist = g_list_append(glist, str);
		}
	}
	return glist;
}

short GetComboNum(short id)
{
	LPCNCLDB lpdb = g_uidb.lpdbTop;
	char *str;
	short i;
	short num = 0;

	for( i = 0 ; i < g_uidb.dbsize ; i++, lpdb++ )
	{
		if( lpdb->nObjectID == id )
		{
			str = ValueToName(lpdb->nObjectID, lpdb->nValue);

			if( *str != '\0' && lpdb->disable == 0 )
				num++;
		}
	}
	return num;
}

char *GetCurrentString(short id)
{
	LPCNCLDB lpdb = g_uidb.lpdbTop;
	char *str = NULL;
	short i;

	for( i = 0 ; i < g_uidb.dbsize ; i++, lpdb++ )
	{
		if( (lpdb->nObjectID == id) && (lpdb->curset == 1) )
		{
			str = ValueToName(lpdb->nObjectID, lpdb->nValue);
			break;
		}
	}
	return str;
}

short GetCurrentnValue(short id)
{
	LPCNCLDB lpdb = g_uidb.lpdbTop;
	short ret = -1;
	short i;

	for( i = 0 ; i < g_uidb.dbsize ; i++, lpdb++ )
	{
		if( (lpdb->nObjectID == id) && (lpdb->curset == 1) )
		{
			ret = lpdb->nValue;
			break;
		}
	}
	return ret;
}

short GetDefaultnValue(short id)
{
	LPCNCLDB lpdb = g_uidb.lpdbTop;
	short ret = -1;
	short i;

	for( i = 0 ; i < g_uidb.dbsize ; i++, lpdb++ )
	{
		if( (lpdb->nObjectID == id) && (lpdb->defset == 1) )
		{
			ret = lpdb->nValue;
			break;
		}
	}
	return ret;
}

short SetTemporaryFlag(short id, short value, short flag)
{
	LPCNCLDB lpdb = g_uidb.lpdbTop;
	short ret = -1;
	short i;

	// Set temporary flag.
	for( i = 0 ; i < g_uidb.dbsize ; i++, lpdb++ )
	{
		if( lpdb->nObjectID == id && lpdb->nValue == value )
		{
			lpdb->tmpset = flag;
			ret = i;
			break;
		}
	}
	return ret;
}

short GetAllFlags(short id, short value)
{
	LPCNCLDB lpdb = g_uidb.lpdbTop;
	short ret = -1;
	short i;

	// Get all flag status.
	for( i = 0 ; i < g_uidb.dbsize ; i++, lpdb++ )
	{
		if( lpdb->nObjectID == id )
		{
			if( lpdb->nValue == value )
			{
				ret = 0;
				if( lpdb->defset )
					ret |= DEFAULT_FLAG;
				if( lpdb->curset )
					ret |= CURRENT_FLAG;
				if( lpdb->tmpset )
					ret |= TEMPSET_FLAG;
				if( lpdb->disable )
					ret |= DISABLE_FLAG;
				break;
			}
		}
	}
	return ret;
}

int UpdateMenuLink(short id, short value)
{
	int ret;
	SetTemporaryFlag(id, value, 1);

	ret = CNCL_GetMenulink(&g_uidb.nameinfo,
			(void*)g_bjlibdir, g_uidb.lpdbTop, g_uidb.dbsize);

#ifdef DEBUG_UPDATE_MENU_LINK
	printf(">>> UpdateMenuLink >>>\n");
	printf("id = %d, value = %d, ret = %d\n", id, value, ret);
#endif

	return ret;
}

void DumpDataBase(int id)
{
	LPCNCLDB lpdb = g_uidb.lpdbTop;
	short i;
	static int count = 0;

	printf("%d >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>\n", count++);

	for( i = 0 ; i < g_uidb.dbsize ; i++, lpdb++ )
	{
		if( id == -1 || id == lpdb->nObjectID )
		{
			printf("dbsize:%d-----------------------------------\n", i);
			printf("nObjectID (CNCLID): %d\n", lpdb->nObjectID);
			printf("nValue    (CNCLID): %d\n", lpdb->nValue);
			printf("defset  : %d\n", lpdb->defset);
			printf("curset  : %d\n", lpdb->curset);
			printf("tmpset  : %d\n", lpdb->tmpset);
			printf("disable : %d\n", lpdb->disable);
		}
	}
}

