/*  Canon Bubble Jet Print Filter.
 *  Copyright CANON INC. 2001
 *  All Rights Reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * NOTE:
 *  - As a special exception, this program is permissible to link with the
 *    libraries released as the binary modules.
 *  - If you write modifications of your own for these programs, it is your
 *    choice whether to permit this exception to apply to your modifications.
 *    If you do not wish that, delete this exception.
 */

/*
 * Initial main.c file generated by Glade. Edit as required.
 * Glade will not overwrite this file.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef ENABLE_NLS
#  include <libintl.h>
#endif

#include <gtk/gtk.h>
#ifdef	USE_LIB_GLADE
#	include <glade/glade.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <popt.h>

#include "callbacks.h"
#ifndef	USE_LIB_GLADE
#	include "interface.h"
#	include "support.h"
#endif


#define	DEFINE_GLOBALS
#include "bjuidefs.h"


void SetGtkResourceFile();
UIDialog* CreateMainWindow();
void InitOption(int argc, char* argv[]);
void FreeOption();
void CreateDialogs();
void DisposeDialogs();
void ConnectSignalHandlers();
void ShowModelDependWidgets();


int main(int argc, char *argv[])
{
	int ret = 1;

	bindtextdomain(PACKAGE, PACKAGE_LOCALE_DIR);
	textdomain(PACKAGE);

	gtk_set_locale();

	// Set gtk resource file.
	SetGtkResourceFile();

	gtk_init(&argc, &argv);

#ifdef	USE_LIB_GLADE
	// Initialize the glade library.
	glade_init();
#endif

	// Parse options.
	InitOption(argc, argv);

	// Set the directory name for storing keytext files.
	SetKeyTextDir(PACKAGE_DATA_DIR);

#ifndef	USE_LIB_GLADE
	// Set the directory name for storing xpm files.
	SetPixmapDir(PACKAGE_DATA_DIR);
#endif

	// Load fontset.
	g_main_font = LoadFontSet();

	// Initialize signal control level.
	InitSignal();

	// Load the glade and keytext file.
	if( LoadResources() )
	{
		// Initialize databases.
		InitDataBase(g_model_name);

		// Create main dialog, and realize it.
		g_main_window = CreateMainWindow();
		gtk_widget_realize(g_main_window->window);

		// Create dialogs.
		CreateDialogs();

		// Connect signal handlers.
		ConnectSignalHandlers();

		// Update widgets.
		DisableSignal();
		UpdateWidgets(g_main_window->window, NULL);
		EnableSignal();

		// Set window title.
		sprintf(g_window_title, "Canon %s", GetModelName());
		gtk_window_set_title(GTK_WINDOW(g_main_window->window), g_window_title);

		// Show widgets depend on model.
		ShowModelDependWidgets();

		gtk_widget_show(g_main_window->window);
		gtk_main();

		ret = 0;

		// Dispose dialogs.
		DisposeDialogs();

		// Dispose main dialog.
		DisposeDialog(g_main_window);

		// Free databases.
		FreeDataBase();

		// Free resources.
		FreeResources();
	}
	// Free Gdk resource.
	gdk_font_unref(g_main_font);

	// Free option strings.
	FreeOption();

	return ret;
}

void SetGtkResourceFile()
{
	// Get gtk resources.
	gchar* home_dir = NULL;
	gchar* rc_path = NULL;

	home_dir = g_get_home_dir();
	rc_path = g_strdup_printf("%s/.gtkrc", home_dir);
	gtk_rc_add_default_file(rc_path);
	g_free(rc_path);
}

UIDialog* CreateMainWindow()
{
	UIDialog* main_window = CreateDialog(sizeof(UIDialog), NULL);

#ifdef	USE_LIB_GLADE
	main_window->window = LookupWidget(NULL, "ui_window");
#else
	main_window->window = create_ui_window();
#endif
	return main_window;
}

#define	UI_OPT_SOCKET	1
#define	UI_OPT_MODEL	2

void InitOption(int argc, char* argv[])
{
	static struct poptOption option_table[] =
	{
		{NULL,   's', POPT_ARG_STRING,  NULL, UI_OPT_SOCKET },
		{"model",  0, POPT_ARG_STRING,  NULL, UI_OPT_MODEL  },
		{NULL,     0,               0,  NULL, 0}
	};

	poptContext context
		 = poptGetContext("printui", argc, (const char**)argv, option_table, 0);
	int rc;

	while( (rc = poptGetNextOpt(context)) > 0 )
	{
		switch( rc )
		{
		case UI_OPT_SOCKET:
			g_socketname = g_strdup(poptGetOptArg(context));
			break;

		case UI_OPT_MODEL:
			g_model_name = g_strdup(poptGetOptArg(context));
			break;

		default:
			break;
		}
	}

	if( rc < -1 )
		g_warning("Unknown switch, %s\n",
			 poptBadOption(context, POPT_BADOPTION_NOALIAS));
}

void FreeOption()
{
	if( g_model_name != NULL )
		g_free(g_model_name);
	if( g_socketname != NULL )
		g_free(g_socketname);
}

void CreateDialogs()
{
	// Create quality dialog.
	g_quality_dialog = CreateQualityDialog(g_main_window);

	// Create color adjustment dialog.
	g_color_dialog = CreateColorDialog(g_main_window);

	// Create user size dialog.
	g_user_size_dialog = CreateUserSizeDialog(g_main_window, g_unit_inch);

	// Create version dialog.
	g_version_dialog = CreateVersionDialog(g_main_window);

	// Create Media & Size dialog.
	g_mediasize_illegal_dialog
		= CreateMediaSizeDialog(g_main_window, UI_MEDIASIZE_DLG_ILLEGAL);
	g_mediasize_recommend_dialog
		= CreateMediaSizeDialog(g_main_window, UI_MEDIASIZE_DLG_RECOMMEND);

	// Create media type dialog.
	g_mediatype_dialog = CreateMediaTypeDialog(g_main_window);
}

void DisposeDialogs()
{
	// Dispose quality dialog.
	DisposeDialog((UIDialog*)g_quality_dialog);

	// Dispose color adjustment dialog.
	DisposeDialog((UIDialog*)g_color_dialog);

	// Dispose user size dialog.
	DisposeDialog((UIDialog*)g_user_size_dialog);

	// Dispose version dialog.
	DisposeDialog((UIDialog*)g_version_dialog);

	// Dispose Media & Size dialog.
	DisposeDialog((UIDialog*)g_mediasize_illegal_dialog);
	DisposeDialog((UIDialog*)g_mediasize_recommend_dialog);

	// Dispose media type dialog.
	DisposeMediaTypeDialog(g_mediatype_dialog);
}

void ConnectSignalHandlers()
{
	GtkWidget* combo;

	combo = LookupWidget(g_main_window->window, "media_type_combo");
	gtk_widget_realize(combo);
	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(combo)->popwin), "event",
			GTK_SIGNAL_FUNC(on_media_type_popwin_event), NULL);

	combo = LookupWidget(g_main_window->window, "media_size_combo");
	gtk_widget_realize(combo);
	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(combo)->popwin), "event",
			GTK_SIGNAL_FUNC(on_media_size_popwin_event), NULL);

	combo = LookupWidget(g_main_window->window, "media_supply_combo");
	gtk_widget_realize(combo);
	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(combo)->popwin), "event",
			GTK_SIGNAL_FUNC(on_media_supply_popwin_event), NULL);

	combo = LookupWidget(g_main_window->window, "cartridge_type_combo");
	gtk_widget_realize(combo);
	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(combo)->popwin), "event",
			GTK_SIGNAL_FUNC(on_cartridge_type_popwin_event), NULL);
}

static
void ShowCartridgeVBox()
{
	// Show or hide cartridge combo and label.
	if( GetComboNum(CNCL_CARTRIDGE) > 1 )
	{
		gtk_widget_show(
			LookupWidget(g_main_window->window, "cartridge_type_vbox"));
	}
	else
	{
		gtk_widget_show(
			LookupWidget(g_main_window->window, "one_cartridge_vbox"));
	}
}

static
void ShowUtilButton(char* button_name)
{
	GtkWidget* button = LookupWidget(g_main_window->window, button_name);
	gtk_widget_show(button);
}

static
void ShowUtilButtons()
{
	short util_flag = GetUtilFlag(g_model_id);

	if( util_flag & UF_CLEANING )
		ShowUtilButton("util_clean_button");
	if( util_flag & UF_DEEP_CLEANING )
		ShowUtilButton("util_refresh_button");
	if( util_flag & UF_ROLLER_CLEANING )
		ShowUtilButton("util_roller_clean_button");
	if( util_flag & UF_NOZZLE_CHECK )
		ShowUtilButton("util_nozzle_check_button");
	if( util_flag & UF_HEAD_ALIGNMENT )
		ShowUtilButton("util_head_adjust_button");
	if( util_flag & UF_POWER_OFF )
		ShowUtilButton("util_power_off_button");
	if( util_flag & UF_AUTO_POWER )
		ShowUtilButton("util_auto_power_button");
	if( util_flag & UF_CUSTOM )
		ShowUtilButton("util_config_button");
}

void ShowModelDependWidgets()
{
	// Show cartridge type box.
	ShowCartridgeVBox();

	// Show utility buttons.
	ShowUtilButtons();
}

